@@ -17,7 +17,8 @@
     use constant _MATH_MPC_T    => 10;
 
 use subs qw( __GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
-             __GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS GMP_LIMB_BITS GMP_NAIL_BITS);
+             __GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS GMP_LIMB_BITS GMP_NAIL_BITS
+             __MATH_GMPz_IV_MAX);
 
 use overload
     '+'    => \&overload_add,
@@ -60,7 +61,8 @@ use overload
 
     @Math::GMPz::EXPORT_OK = qw(
 __GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
-__GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS
+__GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS MATH_GMPz_IV_MAX MATH_GMPz_IV_MIN
+MATH_GMPz_UV_MAX
 Rmpz_abs Rmpz_add Rmpz_add_ui Rmpz_addmul Rmpz_addmul_ui Rmpz_and Rmpz_bin_ui
 Rmpz_bin_uiui Rmpz_bin_si Rmpz_cdiv_q Rmpz_cdiv_q_2exp Rmpz_cdiv_q_ui Rmpz_cdiv_qr
 Rmpz_cdiv_qr_ui Rmpz_cdiv_r Rmpz_cdiv_r_2exp Rmpz_cdiv_r_ui Rmpz_cdiv_ui
@@ -78,6 +80,9 @@ Rmpz_fprintf Rmpz_sprintf Rmpz_snprintf
 Rmpz_gcd Rmpz_gcd_ui Rmpz_gcdext Rmpz_get_d_2exp Rmpz_get_si Rmpz_get_str
 Rmpz_get_ui Rmpz_getlimbn Rmpz_hamdist Rmpz_import Rmpz_init Rmpz_init2
 Rmpz_init2_nobless Rmpz_init_nobless Rmpz_init_set Rmpz_init_set_d
+Rmpz_init_set_IV Rmpz_init_set_NV
+Rmpz_set_IV Rmpz_set_NV Rmpz_cmp_NV
+Rmpz_get_IV Rmpz_fits_IV_p Rmpz_fits_UV_p
 Rmpz_init_set_d_nobless Rmpz_init_set_nobless Rmpz_init_set_si
 Rmpz_init_set_si_nobless Rmpz_init_set_str Rmpz_init_set_str_nobless
 Rmpz_init_set_ui Rmpz_init_set_ui_nobless Rmpz_inp_str Rmpz_inp_raw
@@ -104,7 +109,7 @@ zgmp_randinit_set zgmp_randinit_default_nobless zgmp_randinit_mt_nobless
 zgmp_randinit_lc_2exp_nobless zgmp_randinit_lc_2exp_size_nobless zgmp_randinit_set_nobless
 zgmp_urandomb_ui zgmp_urandomm_ui
     );
-    our $VERSION = '0.43';
+    our $VERSION = '0.44';
     #$VERSION = eval $VERSION;
 
     DynaLoader::bootstrap Math::GMPz $VERSION;
@@ -112,6 +117,7 @@ zgmp_urandomb_ui zgmp_urandomm_ui
     $Math::GMPz::NULL = _Rmpz_NULL();
 
     %Math::GMPz::EXPORT_TAGS =(mpz => [qw(
+MATH_GMPz_IV_MAX MATH_GMPz_IV_MIN MATH_GMPz_UV_MAX
 Rmpz_abs Rmpz_add Rmpz_add_ui Rmpz_addmul Rmpz_addmul_ui Rmpz_and Rmpz_bin_ui
 Rmpz_bin_uiui Rmpz_bin_si Rmpz_cdiv_q Rmpz_cdiv_q_2exp Rmpz_cdiv_q_ui Rmpz_cdiv_qr
 Rmpz_cdiv_qr_ui Rmpz_cdiv_r Rmpz_cdiv_r_2exp Rmpz_cdiv_r_ui Rmpz_cdiv_ui
@@ -132,6 +138,9 @@ Rmpz_init2_nobless Rmpz_init_nobless Rmpz_init_set Rmpz_init_set_d
 Rmpz_init_set_d_nobless Rmpz_init_set_nobless Rmpz_init_set_si
 Rmpz_init_set_si_nobless Rmpz_init_set_str Rmpz_init_set_str_nobless
 Rmpz_init_set_ui Rmpz_init_set_ui_nobless Rmpz_inp_str Rmpz_inp_raw
+Rmpz_init_set_IV Rmpz_init_set_NV
+Rmpz_set_IV Rmpz_set_NV Rmpz_cmp_NV
+Rmpz_get_IV Rmpz_fits_IV_p Rmpz_fits_UV_p
 Rmpz_invert Rmpz_ior new_from_MBI
 Rmpz_jacobi Rmpz_kronecker Rmpz_kronecker_si Rmpz_kronecker_ui Rmpz_lcm
 Rmpz_lcm_ui Rmpz_legendre Rmpz_lucnum2_ui Rmpz_lucnum_ui Rmpz_mod Rmpz_mod_ui
@@ -176,8 +185,10 @@ sub new {
     # if the first arg is a Math::GMPz object (which is a possibility),
     # then we'll get a fatal error when we check it for equivalence to
     # the string "Math::GMPz". So we first need to check that it's not
-    # an object - which we'll do by using the ref() function:
-    if(!ref($_[0]) && $_[0] eq "Math::GMPz") {
+    # an object - which we'll do by using the ref() function.
+    # Also first check that the POK flag is set - to avoid setting the
+    # POK flag on perl-5.18 and earlier.
+    if(!ref($_[0]) && Math::GMPz::_SvPOK($_[0]) && $_[0] eq "Math::GMPz") {
       shift;
       if(!@_) {return Rmpz_init()}
       }
@@ -200,15 +211,12 @@ sub new {
     # Die if there are any additional args (unless $type == _POK_T)
     if($type == _UOK_T || $type == _IOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
-      return Rmpz_init_set_str($arg1, 10);
+      return Rmpz_init_set_IV($arg1);
     }
 
     if($type == _NOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
-      if(Math::GMPz::_has_longdouble()) {
-        return _Rmpz_init_set_ld($arg1);
-        }
-      return Rmpz_init_set_d($arg1);
+      return Rmpz_init_set_NV($arg1);
 
     }
 
@@ -272,6 +280,16 @@ sub TRmpz_out_str {
     die "Wrong number of arguments supplied to TRmpz_out_str()";
 }
 
+# _Rmpz_get_IV may have returned a "string" - in which case we want to coerce it
+# to an IV. It would be more efficient to do this in XS space (TODO), but in the
+# meantime I've taken the soft option of having perl perform the coercion:
+
+sub Rmpz_get_IV {
+   my $ret = _Rmpz_get_IV(shift);
+   $ret += 0 unless _SvIOK($ret); # Set the IV flag
+   return $ret;
+}
+
 sub Rpi_x {
   Rmpz_set_ui($_[0], 1);
   Rmpz_mul_2exp($_[0], $_[0], $_[1]);
@@ -729,7 +747,10 @@ __END__
     Assign the 'unsigned int', $ui, to $rop.
 
    Rmpz_set_d($rop, $double);
-    Assign $double to $rop. (Truncate to an integer if necessary.)
+    Assign $double to $rop. (First truncate to an integer if $double
+    does not contain an integer value.)
+    Program dies with an appropriate error message if $double is
+    Inf or NaN.
 
    Rmpz_set_q($rop, $q); # $q is a Math::GMPq or GMP::Mpq object
     Assign $q to $rop. (Truncate to an integer if necessary.)
@@ -744,12 +765,24 @@ __END__
     hexadecimal is assumed, otherwise if the first character is "0",
     octal is assumed, otherwise decimal is assumed.
 
+   Rmpz_set_IV($rop, $IV) # $IV is a perl integer value
+    Set $rop to the value of $IV. (If $IV is an unsigned integer
+    value then that unsigned value is assigned.)
+
+   Rmpz_set_NV($rop, $NV) # $NV is a perl floating point value
+    Set $rop to the value of $NV. (First truncate to an integer if
+    $NV does not contain an integer value.)
+    The program dies with an appropriate error message if $NV is
+    Inf or NaN.
+
    Rmpz_swap($rop1, $rop2); # swap the values
 
    ######################################
 
    COMBINED INITIALIZATION AND ASSIGNMENT
 
+   Both initialise and set the value in one step.
+
    NOTE: Do NOT use these functions if $rop has already
    been initialised. Instead use the Rmpz_set* functions
    in 'Assigning Integers' (above)
@@ -780,6 +813,13 @@ __END__
    $rop = Rmpz_init_set_str($str, $base);
    $rop = Rmpz_init_set_str_nobless($str, $base);
 
+   $rop = Rmpz_init_set_IV($IV); # $IV is a perl integer value
+   $rop - Rmpz_init_set_NV($NV); # $NV is a perl floating point value
+    Note that Rmpz_init_set_d and Rmpz_init_set_NV will truncate the
+    double/NV to an integer value prior to assigning if the double/NV
+    does not hold an integer value.
+    Assigning an Inf or NaN is a fatal error.
+
    $rop = new_from_MBI($mbi); # $mbi is a Math::BigInt object.
 
    ###################
@@ -790,15 +830,27 @@ __END__
    $ui = Rmpz_get_ui($op);
     Return the value of $op as an `unsigned long'.
     The sign of $op is ignored, only the absolute value is used.
+    If $op is either too big to fit into an `unsigned long int', or
+    is negative, the returned result is probably not very useful.
+    To find out if the value will fit, use the function
+    `Rmpz_fits_ulong_p'.
 
    $si = Rmpz_get_si($op);
     If $op fits into a `signed long int' return the value of $op.
-    Otherwise return the least significant part of OP, with the
+    Otherwise return the least significant part of $op, with the
     same sign as $op. If $op is too big to fit in a `signed long
     int', the returned result is probably not very useful.  To
     find out if the value will fit, use the function
     `Rmpz_fits_slong_p'.
 
+   $IV = Rmpz_get_IV($op);
+    If $op fits into either an IV or a UV return the value of $op.
+    Otherwise return the least significant part of $op, with the
+    same sign as $op. If $op does not fit into either an IV or a
+    UV, the returned result is probably not very useful. To find
+    find out if the value will fit, use the functions
+    'Rmpz_fits_IV_p' and 'Rmpz_fits_UV_p'.
+
    $double = Rmpz_get_d($op);
      Place the value of $op into a normal perl scalar.
 
@@ -1134,11 +1186,16 @@ __END__
 
    $si = Rmpz_cmp($op1, $op2);
    $si = Rmpz_cmp_d($op, $double);
+   $si = Rmpz_cmp_NV($op, $nv); # $nv is an NV
    $si = Rmpz_cmp_si($op, $si);
    $si = Rmpz_cmp_ui($op, $ui);
     Compare 1st and 2nd args.  Return a positive value if
     1st arg > 2nd arg, zero if 1st arg = 2nd arg, or a
     negative value if 1st arg < 2nd arg.
+    Note that Rmpz_cmp_d and Rmpz_cmp_NV compare $op to the
+    actual value held in the double/NV - not to the truncated
+    integer value. Comparison to an Inf is allowed, but
+    comparison to a NaN is a fatal error.
 
    $si = Rmpz_cmpabs($op1, $op2);
    $si = Rmpz_cmpabs_d($op, $double);
@@ -1404,17 +1461,21 @@ __END__
 
    MISCELLANEOUS INTEGER FUNCTIONS
 
-   $bool = Rmpz_fits_ulong_p($op);
-   $bool = Rmpz_fits_slong_p($op);
-   $bool = Rmpz_fits_uint_p($op);
-   $bool = Rmpz_fits_sint_p($op);
+   $bool = Rmpz_fits_ulong_p ($op);
+   $bool = Rmpz_fits_slong_p ($op);
+   $bool = Rmpz_fits_uint_p  ($op);
+   $bool = Rmpz_fits_sint_p  ($op);
    $bool = Rmpz_fits_ushort_p($op);
    $bool = Rmpz_fits_sshort_p($op);
-    Return non-zero iff the value of $op fits an `unsigned long int',
-    `signed long int', `unsigned int', `signed int', `unsigned short
-    int', or `signed short int', respectively. Otherwise, return zero.
+   $bool = Rmpz_fits_UV_p    ($op);
+   $bool = Rmpz_fits_IV_p    ($op);
 
-   $bool = Rmpz_odd_p($op);
+    Return non-zero iff the value of $op fits an 'unsigned long int',
+    'signed long int', 'unsigned int', 'signed int', 'unsigned short
+    int', or 'signed short int', 'UV', or 'IV' respectively.
+    Otherwise, return zero.
+
+   $bool = Rmpz_odd_p ($op);
    $bool = Rmpz_even_p($op);
     Determine whether $op is odd or even, respectively.
     Return non-zero if yes, zero if no.
